import java.net.*;
import java.io.*;
import enos.scrabble.domain.*;

public class EnosScrabbleClient {

  // The client trustes the server to respect the protocol
  // (and the rules of the game, of course)
  // So the client is NOT required to do any checking on the input messages
  // This makes the client code very clean  compared to the server code,
  // which is a mess (it tries to check any possible protocol error)
  //
  // Notes: in my internal d/s, players range from 0 to noplayers - 1
  //        this explains those " -1" below

  public static void main(String[] args) {

    String servername = "";
    int serverport = 20000;

    ScrabbleWorld world;

    int noplayers = 2;
    int myplayno;

    Socket mysock;
    PrintWriter playw; // extends BufferedWriter
    BufferedReader playr;

    Game theGame = Game.singleton();

    // parse command line
    int i = 0;
    while (i < args.length && args[i].startsWith("-")) {
      String arg = args[i++];
      if (arg.equals("-server")) {
        servername = args[i++];
      }
      else if (arg.equals("-port")) {
        serverport = Integer.parseInt(args[i++]);
      }
    }

    try {

      My.pr("Attempting connection to " + servername + ":" + serverport);
      // connects to the server
      mysock = new Socket(servername, serverport);
      playw = new PrintWriter(mysock.getOutputStream());
      playr = new BufferedReader(new InputStreamReader(mysock.getInputStream()));

      // wait for the "YOUARE" message
      // this is typically the first message send by the server
      // but the server can also start the protocol with several
      // INFO fields, which I'll print for debugging

      String l;
      do {
        l = playr.readLine();
        if (l.startsWith("INFO")) {
          My.pr(l);
        }
      }
      while (!l.startsWith("YOUARE"));

      // parse the YOUARE message
      myplayno = Integer.parseInt(playr.readLine()) - 1;
      noplayers = Integer.parseInt(playr.readLine());

      My.pr("Connection established. I am player " + (myplayno + 1) + "/" +
            noplayers);

      // You may want to comment the following line
      if (noplayers != 2) {
        My.err("I don't know to play with more than 1 opponent");

        // reads the dictionary and generates
        // the internal "refference" state

//*********world=new ScrabbleWorld(false, "ospd.txt","goodies.txt","letters.txt", noplayers);

         // While the game is active wait messages for the server
         // Most messages give you info, so you can update your refference state
         // PLAY is the only message you have to reply to

      }
      boolean gameactive = true;
      int p; // this is the player we are talking about
      while (gameactive) {
        l = playr.readLine();
        // My.debug(l);
        //
        // NEWLETTERS message
        // some player (may be us) received some characters
        //
        if (l.startsWith("NEWLETTERS")) {
          p = Integer.parseInt(playr.readLine()) - 1;
          String letters = playr.readLine();
//**************    world.curr_state.fillhand(p, letters);

          theGame.giveLettersToPlayer(letters, p);
          My.p("\nPlayer <" + (p+1) + "> rack is now: " +
               theGame.readRack(p) + "\n");
          // this would be a good place to findt the potential perms
          // for the next move
          //
          // VALIDMOVE MESSAGE
          // some player (may be us) made a valid move
          //
        }
        else if (l.startsWith("VALIDMOVE")) {
          p = Integer.parseInt(playr.readLine()) - 1;
// *******          boolean movedir = playr.readLine().startsWith("VER");
          Move.Direction direction = playr.readLine().startsWith("VER")
              ? Move.VERT
              : Move.HORIZ;
          int movex = Integer.parseInt(playr.readLine());
          int movey = Integer.parseInt(playr.readLine());
          String moveword = playr.readLine();
          int movescore = Integer.parseInt(playr.readLine());
//*********		    ScrabbleMove move = new ScrabbleMove(moveword, movex, movey, movedir,
//*******						 movescore);
//******		    world.curr_state.do_move(move, p);
             Move theMove = new Move(new Word(moveword), direction,
                                     movex, movey);
          My.p("\nPlayer <" + (p + 1) + "> played:\n");
          My.p(theMove.toString());
          if (0 == movex && 0 == movey && moveword.equals("")) {
            // pass
          } else {
            try {
              theGame.move(theMove, p);
            }
            catch (InvalidMoveException e) {
              My.p("CHALLENGE!*********************\n" +
                     "WORD --> " + theMove.getString() + "\n" +
                     "I declare my opponent's move invalid " +
                     "for the following reason:");
              My.p(e.getMessage());
              e.printStackTrace(System.out);
              System.out.println(e.getMessage());
              System.exit(1);
            }
          }
          //
          // ENDOFGAME message
          // ok, some score corrections
          // and then exit the main loop
          //
        }
        else if (l.startsWith("ENDOFGAME")) {
          for (p = 0; p < noplayers; p++) {
            int endcorrscore = Integer.parseInt(playr.readLine());
//			world.curr_state.scores[p] += endcorrscore;
            My.pr("The server is trying to adjust player " + (int) (p + 1) +
                  "\'s score by " + endcorrscore + " points. " +
                  "I'm a little suspicious.");
          }
          gameactive = false;

          //
          // PLAY message
          // now we have to find a good move
          // and send it to the server
          //
        }
        else if (l.startsWith("PLAY")) {
          // it's a good idea to play with a copy of
          // the refference state instead of using
          // the original copy

          long timer = System.currentTimeMillis();

          /*		    ScrabbleState ss = new ScrabbleState(world.curr_state);
                PlayerMove pm = new PlayerMove(world, ss, myplayno);
                pm.trymoves(2, 1000000);
                ScrabbleMove mymove = pm.bestmove;
                playw.println("MOVE");
                playw.println(mymove.dir ? "VER" : "HOR");
                playw.println(mymove.getx());
                playw.println(mymove.gety());
                playw.println(mymove.getWord());
                playw.flush();
                timer = System.currentTimeMillis() - timer;
                My.p("timer="+timer+"ms\t");
                mymove.print();
           */
          Move theMove = theGame.getMove(myplayno);

          if (null == theMove) {
            playw.println("MOVE");
            My.pr("passing*********");
            My.pr("MOVE");
            playw.println("HOR");
            My.pr("HOR");
            playw.println(0);
            My.pr(0 + "");
            playw.println(0);
            My.pr(0 + "");
            playw.println("");
            My.pr("");
          }
          else {
            playw.println("MOVE");
            My.pr("MOVE");
            playw.println(theMove.getDirection() == Move.VERT
                          ? "VER" : "HOR");
            My.pr(theMove.getDirection() == Move.VERT
                  ? "VER" : "HOR");
            playw.println(theMove.getX());
            My.pr(theMove.getX() + "");
            playw.println(theMove.getY());
            My.pr(theMove.getY() + "");
            playw.println(theMove.getString());
            My.pr(theMove.getString());
          }
          playw.flush();
          timer = System.currentTimeMillis() - timer;
          My.p("timer=" + timer + "ms\t \n");
//              mymove.print();

        }
        else { // this has to be an INFO message
          My.pr(l);
        }

      }

      mysock.close();
      My.pr("Connection closed. Bye");

    }
    catch (IOException e) {
      My.err("Client error", e);
    }

  }

}