package enos.scrabble.datastruct;

import java.util.*;
import enos.scrabble.domain.Word;

public class HashOfWords {
  private int nextBucketIndex = 0;
  private short[] indices;
  ArrayList[] buckets = new ArrayList[65534];

  public HashOfWords() {
    indices = new short[67108863];
    Arrays.fill(indices, (short)-32768);
  }

  public void add(Word theWord) {
    int hash = theWord.hashBinary();
    int theIndex = index(hash);
    ArrayList theBucket = null;
    // lazy init the bucket - we don't really want 65534 ArrayLists
    if (-1 == theIndex) {
      theIndex = assignIndex(hash);
      theBucket = new ArrayList();
      buckets[theIndex] = theBucket;
    }else {
      theBucket = buckets[theIndex];
    }
    if (!theBucket.contains(theWord))
      theBucket.add(theWord);
  }

  public boolean exists(Word theWord) {
    List bucket = getBucket(theWord);
    if (null == bucket)
      return false;
    else
      return bucket.contains(theWord);
  }

  private int assignIndex(int hash) {
      int index = nextBucketIndex++;
      int shortIndex = index - 32767;
      indices[hash] = (short)shortIndex;
      return index;
  }

  private int index(int hash) {
    int theIndex = indices[hash];
    theIndex += 32767;
    return theIndex;
  }

  public List getMatches(Word theWord) {
    int hash = theWord.hashBinary();
    int theIndex = index(hash);
    if (-1 == theIndex) {
      return null;
    } else {
      List theList = new ArrayList();
      Iterator iter = buckets[theIndex].iterator();
      while (iter.hasNext()) {
        Word nextWord = (Word)iter.next();
        if (theWord.sameLetters(nextWord)) {
          if(theWord.hasAnchor()) {
             if (nextWord.containsSubstring(theWord.getAnchorLetters())) {
               nextWord.synchronizeCaps(theWord);
               nextWord.addAnchor(theWord.getAnchor());
               theList.add(nextWord);
             }
          }else {
            nextWord.synchronizeCaps(theWord);
            theList.add(nextWord);
          }
        }
      }
      return theList;
    }
  }

  public List getBucket(Word theWord) {
    int hash = theWord.hashBinary();
    int theIndex = index(hash);
    if (-1 == theIndex) {
      return null;
    } else {
      return buckets[theIndex];
    }
  }
}
