package enos.scrabble.domain;

import java.io.*;
import java.util.*;
import java.net.URL;

public class Board {
  private Square[][] squares = new Square[15][15];
  private char[][] playedLetts = new char[15][15]; // y first, then x !
  boolean isFirstPlay = true;

  public Board() {
    try {
      init();
    }
    catch (IOException ex) {
      System.out.println("unable to initialize board.");
      ex.printStackTrace();
      System.exit(1);
    }
  }

  private Board(Square[][] theSquares, char[][] letters) {
    squares = theSquares;
    playedLetts = letters;
  }

  public Object clone() {
    char[][] newLetters = new char[15][15];
    for (int i = 0; i < playedLetts.length; i++) {
      newLetters[i] = (char[]) playedLetts[i].clone();
    }
    Square[][] newSquares = new Square[15][15];
    for (int i = 0; i < squares.length; i++) {
      for (int j = 0; j < squares[i].length; j++) {
        newSquares[i][j] = (Square) squares[i][j].clone();
      }
    }
    Board newBoard = new Board(newSquares, newLetters);
    newBoard.isFirstPlay = this.isFirstPlay;
    return newBoard;
  }

  public void executeMove(MoveGroup group) throws InvalidMoveException {
    isFirstPlay = false;
    int numberPlayed = 0;
    Move theMove = group.originalMove();
    int x = theMove.getX();
    int y = theMove.getY();
    Move.Direction direct = theMove.getDirection();
    try {
      while (theMove.hasNextLetter()) {
        char letter = theMove.nextLetter();
        if (playedLetts[y][x] != 0) {
          if (playedLetts[y][x] != letter) {
            throw new InvalidMoveException(
                "Attempted to replace existing letter!");
          }
        }
        else {
          playedLetts[y][x] = letter;
          numberPlayed++;
        }
//        System.out.print(letter);
        if (direct.equals(Move.HORIZ)) {
          x++;
        }
        else {
          y++;
        }
      }
//      System.out.println();
//      System.out.println("played->" + numberPlayed);
    }
    catch (ArrayIndexOutOfBoundsException aie) {
      throw new InvalidMoveException(
          "Attempted to place letter off the board!", aie);
    }
    theMove.signalSquaresUsed();
//    printBoard();
  }

  public boolean isValid(Move theMove) {
    int x = theMove.getX();
    int y = theMove.getY();
    int countOfNewLetters = 0;
    Move.Direction direct = theMove.getDirection();
    try {
      while (theMove.hasNextLetter()) {
        char letter = theMove.nextLetter();
        if (playedLetts[y][x] != 0) {
          if (playedLetts[y][x] != letter) {
            return false;
          }
        } else {
          countOfNewLetters++;
        }
        //System.out.print(letter);
        if (direct.equals(Move.HORIZ)) {
          x++;
        }
        else {
          y++;
        }
      }
    }
    catch (ArrayIndexOutOfBoundsException aie) {
      return false;
    }
    if (7 == countOfNewLetters) theMove.signalUsesSeven();
    return true;
  }

  public int numberOfNewLettersIn(Move theMove) {
    int length = theMove.getString().length();
    Move.Direction dir = theMove.getDirection();
    int x = theMove.getX();
    int y = theMove.getY();
    int count = 0;
    if (Move.HORIZ == dir) {
      for (int i = 0; i < x + length; i++) {
        if (0 == playedLetts[y][i+x]) {
          count++;
        }
      }
    }else {
      for (int i = 0; i < y + length; i++) {
        if (0 == playedLetts[i+y][x]) {
          count++;
        }
      }
    }
    return count;
  }

  public MoveGroup getCollateralMoves(Move theMove) throws InvalidMoveException {
    try {
      if ( theMove.getScore() <= 0) {
        setScore(theMove);
      }
      if (theMove.getDirection() == Move.VERT) {
        List collateralMoves = checkVerticalMove(theMove);
        return new MoveGroup(theMove, collateralMoves);
      }
      else {
        List collateralMoves = checkHorizontalMove(theMove);
        return new MoveGroup(theMove, collateralMoves);
      }
    } catch (ArrayIndexOutOfBoundsException e) {
      InvalidMoveException im =
          new InvalidMoveException("Picked the wrong place for anchor",e);
      im.fillInStackTrace();
      throw im;
    }
  }

  /** VERT:
       * 1) check above and below, if move extends up & down, is it still valid word
   *     if valid add move to list
   * 2) for each letter not in anchor
   *    check left, right for adjacent letters; is it a valid word
   *    if valid add to list
   */
  private List checkVerticalMove(Move theMove) throws InvalidMoveException {
    List collateralMoves = new ArrayList();
    int startX = theMove.getX();
    int startY = theMove.getY();
    Move aMove = extendVertically(theMove.getString(), startX, startY);
    if (aMove != null) {
      setScore(theMove);
      collateralMoves.add(aMove);
    } else {
      collateralMoves.add(theMove);
    }
    setLettersUsed(theMove);
    int currentY = startY;
    while (theMove.hasNextLetter()) {
      char let = theMove.nextLetter();
      if (0 == playedLetts[currentY][startX]) {
        aMove = extendHorizontally(let + "", startX, currentY);
        if (aMove != null) {
          setScore(aMove);
          collateralMoves.add(aMove);
        }
      }
      currentY++;
    }
    return collateralMoves;
  }

  /**
   * HORIZ:
   * 1) check left and right, if move extends l & r, is it still valid word
   *     if valid add move to list
   * 2) for each letter not in anchor
   *    check above, below for adjacent letters; is it a valid word
   *    if valid add to list
   */
  private List checkHorizontalMove(Move theMove) throws InvalidMoveException {
    List collateralMoves = new ArrayList();
    int startX = theMove.getX();
    int startY = theMove.getY();
    Move aMove = extendHorizontally(theMove.getString(), startX, startY);
    if (aMove != null) {
      setScore(theMove);
      collateralMoves.add(aMove);
    } else {
      collateralMoves.add(theMove);
    }
    setLettersUsed(theMove);
    int currentX = startX;
    while (theMove.hasNextLetter()) {
      char let = theMove.nextLetter();
      if (0 == playedLetts[startY][currentX]) {
        aMove = extendVertically(let + "", currentX, startY);
        if (aMove != null) {
          setScore(aMove);
          collateralMoves.add(aMove);
        }
      }
      currentX++;
    }
    return collateralMoves;
  }

  private Move extendVertically(String letters, int startX, int startY) throws
      InvalidMoveException {
    Move theMove = null;
    int lettersAbove = 0;
    int length = letters.length();
    int extendedStartY = startY;
    StringBuffer buf = new StringBuffer(letters);
    int indexAbove = startY - 1;
    while (indexAbove > -1 && playedLetts[indexAbove][startX] != 0 ) {
      buf.insert(0, playedLetts[indexAbove][startX]);
      extendedStartY = --indexAbove;
    }
    int indexBelow = startY + length;
    while (indexBelow < 15 && playedLetts[indexBelow][startX] != 0) {
      buf.append(playedLetts[indexBelow][startX]);
      indexBelow++;
    }
    if (!letters.equals(buf.toString())) {
      if (!Dictionary.singleton().exists(buf.toString())) {
        InvalidMoveException im =
            new InvalidMoveException("Word: " + buf.toString() +
                                       " not in dictionary.");
        im.fillInStackTrace();
        throw im;
      }
      else {
        theMove = new Move(new Word(buf.toString()), Move.VERT,
                           startX, extendedStartY+1);
      }
    }
    return theMove;
  }

  private Move extendHorizontally(String letters, int startX, int startY) throws
      InvalidMoveException {
    Move theMove = null;
    int lettersToLeft = 0;
    int length = letters.length();
    int extendedStartX = startX;
    StringBuffer buf = new StringBuffer(letters);
    int indexToLeft = startX - 1;
    while (indexToLeft > -1 && playedLetts[startY][indexToLeft] != 0 ) {
      buf.insert(0, playedLetts[startY][indexToLeft]);
      extendedStartX = --indexToLeft;
    }
    int indexToRight = startX + length;
    while (indexToRight < 15 && playedLetts[startY][indexToRight] != 0 ) {
      buf.append(playedLetts[startY][indexToRight]);
      indexToRight++;
    }
    if (!letters.equals(buf.toString())) {
      if (!Dictionary.singleton().exists(buf.toString())) {
        InvalidMoveException im =
            new InvalidMoveException("Word: " + buf.toString() +
                                       " not in dictionary.");
        im.fillInStackTrace();
        throw im;
      }
      else {
        theMove = new Move(new Word(buf.toString()), Move.HORIZ,
                           extendedStartX+1, startY);
      }
    }
    return theMove;
  }

  private void setLettersUsed(Move theMove) {
    StringBuffer buf = new StringBuffer();
    int x = theMove.getX();
    int y = theMove.getY();
    String str = theMove.getString();
    int len = str.length();
    if (theMove.getDirection() == Move.HORIZ) {
      for (int i = 0; i < len; i++) {
        if (0 == playedLetts[y][x + i]) {
          if (str.charAt(i) < 'Z') {
            buf.append("{");
          }else {
            buf.append(str.charAt(i));
          }
        }
      }
    }
    else { // HORIZ
      for (int i = 0; i < len; i++) {
        if (0 == playedLetts[y + i][x]) {
          if (str.charAt(i) < 'Z') {
            buf.append("{");
          }else {
            buf.append(str.charAt(i));
          }
        }
      }
    }
    theMove.setLettersUsed(buf.toString());
  }

  private boolean exists(String str) {
    return Dictionary.singleton().exists(str);
  }

  public int setScore(Move theMove) {
    int x = theMove.getX();
    int y = theMove.getY();
    Move.Direction direct = theMove.getDirection();
    Stack wordBonuses = null;
    List squaresUsed = new ArrayList();
    while (theMove.hasNextLetter() && x < 15 && y < 15) {
      Square theSquare = squares[y][x];
      squaresUsed.add(theSquare);
      if (theSquare instanceof WordBonusSquare) {
        if (null == wordBonuses) {
          wordBonuses = new Stack();
        }
        wordBonuses.push(theSquare);
      }
      char letter = theMove.nextLetter();
      //System.out.print(letter);
      theMove.increaseScoreBy(theSquare.letterScore(letter));
      if (direct.equals(Move.HORIZ)) {
        x++;
      }
      else {
        y++;
      }
    }
    if (wordBonuses != null) {
      while (!wordBonuses.isEmpty()) {
        Square theSquare = (Square) wordBonuses.pop();
        theMove.addWordBonus(theSquare);
      }
    }
    theMove.setSquaresUsed(squaresUsed);
    return theMove.getScore();
  }

  public List getAnchors() {
    List anchors = new ArrayList();
    if (isFirstPlay) {
      return anchors;
    } else {
      // horizontal
      for (int y = 0; y < playedLetts.length; y++) {
        for (int x = 0; x < playedLetts[y].length; x++) {
          if (playedLetts[y][x] != 0) {
            int startX = x;
            int startY = y;
            StringBuffer buf = new StringBuffer();
            while (x < 15 && playedLetts[y][x] != 0) {
              buf.append(playedLetts[y][x++]);
            }
            anchors.add(new Anchor(buf.toString(), startX, startY, Move.HORIZ));
          }
        }
      }
      // vertical
      for (int x = 0; x < 15; x++) {
        for (int y = 0; y < 15; y++) {
          if (playedLetts[y][x] != 0) {
            int startX = x;
            int startY = y;
            StringBuffer buf = new StringBuffer();
            while (y < 15 && playedLetts[y][x] != 0) {
              buf.append(playedLetts[y++][x]);
            }
            anchors.add(new Anchor(buf.toString(), startX, startY, Move.VERT));
          }
        }
      }
      return anchors;
    }
  }

  private void printBoard() {
    System.out.println();
    for (int i = 0; i < 15; i++) {
      for (int j = 0; j < 15; j++) {
        if (playedLetts[i][j] != 0)
          System.out.print(playedLetts[i][j] + " ");
        else
          System.out.print("_ ");
      }
      System.out.println();
    }
  }

  public void replaceLetters(char[][] letts) {
    // for testing purposes only
    playedLetts = letts;
    isFirstPlay = false;
  }

  private void init() throws IOException {
    BufferedReader reader = null;
    File file = new File(Game.boardFile);
    if (file.exists()) {
      reader = new BufferedReader(new FileReader(file));
//      System.out.println("as file");
    }else {
      InputStream str = getClass().getResourceAsStream("/" + Game.boardFile);
      InputStreamReader isReader = new InputStreamReader(str);
      reader = new BufferedReader(isReader);
    }
    String line = null;
    int i = 0;
      while (null != (line = reader.readLine()) && i < 15) {
      StringTokenizer tok = new StringTokenizer(line);
      for (int j = 0; j < 15; j++) {
        String type = tok.nextToken();
        squares[i][j] = makeSquare(type);
      }
      i++;
    }
    reader.close();
  }

  private Square makeSquare(String type) {
    if (type.equals("_")) {
      return new PlainSquare();
    }
    else if (type.equals("3W")) {
      return new WordBonusSquare(3);
    }
    else if (type.equals("2W")) {
      return new WordBonusSquare(2);
    }
    else if (type.equals("3L")) {
      return new LetterBonusSquare(3);
    }
    else if (type.equals("2L")) {
      return new LetterBonusSquare(2);
    }
    else {
      throw new RuntimeException("Bad data in board file");
    }
  }

  public static void main(String[] args) {
    Board originalBoard = null;
    originalBoard = new Board();

    char[][] b = originalBoard.playedLetts;
    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't';
    b[7][8] = 'h';
    b[7][9] = 'e';
    b[8][7] = 'e';
    b[8][9] = 'n';
    b[9][9] = 'd';
    Board theBoard = (Board) originalBoard.clone();
    List theAnchors = theBoard.getAnchors();
    Iterator iter = theAnchors.iterator();
    while (iter.hasNext()) {
      System.out.println(iter.next());
    }
    for (int i = 0; i < 15; i++) {
      for (int j = 0; j < 15; j++) {
        System.out.print(theBoard.squares[i][j] + "  ");
      }
      System.out.println();
    }
  }
}