package enos.scrabble.domain;

import java.io.*;
import java.util.*;
import java.net.URL;
import enos.scrabble.util.*;

public class Game {
  private static Game singleton;
  public static final int PLAYER_1 = 0;
  public static final int PLAYER_2 = 1;
  public static final String boardFile = "board.txt";
  public static final String letterFile = "letters.txt";
  public static final String dictionaryFile = "ospd.txt";
  public static final String bigramFile = "bigram.txt";
  public static final char BLANK = '{';
  private static int[] letterScores = new int[27];
  public static boolean isHosed = false;

  private Board theBoard = new Board();
  private Player[] players = new Player[2];
  private int[] playerScores = new int[2];
  private int[] bag = new int[27];
  private boolean bagIsEmpty = false;

  static { singleton = new Game();
    singleton.init();
  }

  public Object clone() {
    Board newBoard = (Board)theBoard.clone();
    Player newP1 = (Player)players[PLAYER_1].clone();
    Player newP2 = (Player)players[PLAYER_2].clone();
    int[] newBag = (int[])bag.clone();
    return new Game(newBoard, newP1, newP2, newBag, bagIsEmpty);
  }

/**
 * game goes like this:
 * 1> Player 1 receives tiles - tiles removed from bag;
 * 2> Player 1 receives tiles - tiles removed from bag;
 *
 * 3> Player 1 selects best word from rack.
 * 4> Player 1 plays word, updates score.
 * 5> Player 1 removes word from rack.
 * 6> Player 2 notified of word, updates board.
 * 7> Player 1 receives new tiles - tiles removed from bag.
 * 8> repeat...
 */

  public static Game singleton() {
    return singleton;
  }

  private Game(Board aBoard, Player p1, Player p2, int[] newBag, boolean empty) {
    theBoard = aBoard;
    players[PLAYER_1] = p1;
    players[PLAYER_2] = p2;
    bag = newBag;
    bagIsEmpty = empty;
  }

  private Game() {
    players[PLAYER_1] = new Player();
    players[PLAYER_2] = new Player();
  }

//  public Move getMove(int player) {
//    MoveGroup best = getBestMoveGroup(player);
//    if (null == best) {
//      return null;
//    } else {
//      return best.originalMove();
//    }
//  }

  public Move getMove(int player) {
    MiniMaximizer mm = new MiniMaximizer();
    if (PLAYER_1 == player) {
      if (bagIsEmpty) {
        return mm.miniMax(this, PLAYER_1, PLAYER_2, 4);
      } else {
        return mm.miniMax(this, PLAYER_1, PLAYER_2, 2);
      }
    } else {
      if (bagIsEmpty) {
        return mm.miniMax(this, PLAYER_2, PLAYER_1, 4);
      } else {
        return mm.miniMax(this, PLAYER_2, PLAYER_1, 2);
      }
    }
  }

  public MoveGroup getBestMoveGroup(int player) {
    return players[player].getBestMove(theBoard);
  }

  public List getNbestMoveGroups(int player, int n) {
    return players[player].bestNgroups(theBoard, n);
  }

  public void move(Move theMove, int player) throws InvalidMoveException {
    MoveGroup group = null;
    try {
      group = theBoard.getCollateralMoves(theMove);
    }catch (InvalidMoveException e) {
        System.out.println("Choked on collateral moves.");
        throw e;
    }
    move(group, player);
  }

  public void move(MoveGroup group, int player) throws InvalidMoveException {
    theBoard.executeMove(group);
    playerScores[player] += group.score();
//    System.out.println("Player <" + (player +1) + "> score: " + playerScores[player] +
//                       "for word: " + group.originalMove().getString());
//    System.out.println("move score --> " + group.score());
    Move theMove = group.originalMove();
    theMove.signalSquaresUsed();
    players[player].useLetters(theMove);
  }

  public int numLettersNeeded(int player) {
    return players[player].numLettersNeeded();
  }

  public void giveLettersToPlayer(String letters, int player) {
    players[player].receiveLetters(letters.replaceAll("\\*", "{"));
    removeFromBag(letters);
  }

  public boolean bagIsEmpty() {
    return bagIsEmpty;
  }

  public static int letterScore(char letter) {
    if (letter < 'a' || letter > 'z')
      return 0;
    else
      return letterScores[letter - 'a'];
  }

  public String readRack(int player) {
    return players[player].readRack();
  }

  private void removeFromBag(String letts) {
    char[] letters = letts.toCharArray();
    for (int i = 0; i < letters.length; i++) {
      if (letters[i] == '*' || letters[i] == '{') {
        bag[26]--;
      }
      else
      if (letters[i] != 0) {
        bag[letters[i] - 'a']--;
      }
    }
  }

  public String getNletters(int n) {
    if (!bagIsEmpty) {
    Random rand = new Random();
    StringBuffer buf = new StringBuffer();
    for (int i = 0; i < n; i++ ) {
      if (!bagIsEmpty) {
        int[] indices = new int[27];
        int upperIndex = 0;
        for (int j = 0; j < bag.length; j++) {
          if (bag[j] > 0) {
            indices[upperIndex] = j;
            upperIndex++;
          }
        }
        if (0 == upperIndex) {
          bagIsEmpty = true;
        }
        else {
          int pointer = rand.nextInt(upperIndex); //exclusive upper bound
          int index = indices[pointer];
          buf.append( (char) (index + 'a'));
          bag[index]--;
        }
      }
    }
    return buf.toString();
  }
  else {
    return null;
  }
//    Random rand = new Random();
//    StringBuffer buf = new StringBuffer();
//    for (int i = 0;i < n ; i++ ) {
//      int index = rand.nextInt(27);
//      while (bag[index] < 1) {
//        index = rand.nextInt(27);
//      }
//      buf.append((char)(index + 'a'));
//      bag[index]--;
//    }
//    return buf.toString();
  }

  private void init() {
    BufferedReader reader = null;
    try {
      File file = new File(letterFile);
      if (file.exists()) {
        reader = new BufferedReader(new FileReader(file));
      }
      else {
        URL url = getClass().getResource("/" + letterFile);
        InputStreamReader isReader = new InputStreamReader(url.openStream());
        reader = new BufferedReader(isReader);
      }
      String line = null;
      for (int i = 0; i < 27; i++) {
        line = reader.readLine();
        StringTokenizer tok = new StringTokenizer(line);
        String letter = tok.nextToken();
        bag[i] = Integer.parseInt(tok.nextToken());
        letterScores[i] = Integer.parseInt(tok.nextToken());
      }
    }
    catch (IOException ex) {
      System.out.println("Unable to initialize game.");
      ex.printStackTrace();
      System.exit(1);
    }

  }

  public static void main(String[] args) {
    Game theGame = Game.singleton();
    for (int i = 0; i < 27; i++) {
      System.out.println(((char)('a' + i)) + "\t" + theGame.bag[i] +
                         "\t" + theGame.letterScores[i]);
    }
  }
}