package enos.scrabble.domain;

import java.util.List;
import java.util.Iterator;

public class Move {
  public static final Direction HORIZ = new Direction(1);
  public static final Direction VERT = new Direction(2);
  private Word theWord;
  private Move.Direction theDirection;
  private int xCoord, yCoord;
  private int score = 0;
  private List squaresUsed = null;
  private MoveIterator it = null;
  private boolean usesSeven = false;

  public Move(Word word, Move.Direction direct, int initX, int initY) {
    theWord = word;
    theDirection = direct;
    xCoord = initX;
    yCoord = initY;
  }

  public Move(Word word) {
    if (!word.hasAnchor())
      throw new RuntimeException("Use of this constructor requires that Word " +
                                 " contain an Anchor.");
    theWord = word;
    Anchor theAnchor = theWord.getAnchor();
    theDirection = theAnchor.getDirection();
    if (Move.HORIZ == theDirection) {
      yCoord = theAnchor.startY();
      int anchLoc = theWord.indexOfAnchor();
      xCoord = theAnchor.startX() - anchLoc;
    } else { // VERT
      xCoord = theAnchor.startX();
      int anchLoc = theWord.indexOfAnchor();
      yCoord = theAnchor.startY() - anchLoc;
    }
  }

  public void signalSquaresUsed() {
    if (squaresUsed != null) {
      Iterator iter = squaresUsed.iterator();
      while (iter.hasNext()) {
        Square theSquare = (Square) iter.next();
        theSquare.signalUsed();
      }
    }
  }

  public void setSquaresUsed(List squares) {
    squaresUsed = squares;
  }

  public void setLettersUsed(String letters) {
    theWord.setLettersUsed(letters);
  }

  public String newLettersUsed() {
    return theWord.newLettersUsed();
  }

  public int getX() {
    return xCoord;
  }

  public int getY() {
    return yCoord;
  }

  public int length() {
    return theWord.length();
  }

  public String getString() {
    return theWord.toString();
  }

  public Direction getDirection() {
    return theDirection;
  }

  public void signalUsesSeven() {
    usesSeven = true;
  }

  public int getScore() {
    if (usesSeven) {
      return score + 50;
    } else {
      return score;
    }
  }

  public void increaseScoreBy(int increase) {
    score += increase;
  }

  public void addWordBonus(Square theSquare) {
    score = theSquare.wordScore(this);
  }

  public boolean hasNextLetter() {
    if (null == it) it =  new MoveIterator(theWord.toString());
    if (it.hasNext()) {
      return true;
    } else {
      it = null;
      return false;
    }
  }

  public char nextLetter() {
    return it.nextChar();
  }

  private class MoveIterator {
    private char[] chars;
    int nextIndex = 0;
    private MoveIterator(String str) {
      chars = (str.toLowerCase()).toCharArray();
    }

    private boolean hasNext() {
      if (nextIndex < chars.length)
        return true;
      else
        return false;
    }

    private char nextChar() {
      if (nextIndex < chars.length) {
        return chars[nextIndex++];
      }
      else {
        throw new ArrayIndexOutOfBoundsException(
            "called next char too many times");
      }
    }
  }

  public String toString() {
    return (this.getString() + "\n" +
            this.getDirection().toString() + "\n" +
            "X coord-> " + this.getX() + "\n" +
            "Y coord-> " + this.getY() + "\n");
  }

    public static class Direction {
      int code;
      private Direction(int theCode) {
        code = theCode;
      }

      public boolean equals(Object other) {
        if (other == this) {
          return true;
        }
        if (! (other instanceof Direction)) {
          return false;
        }
        Direction that = (Direction) other;
        if (this.code == that.code)
          return true;
        else
          return false;
      }

      public String toString() {
        if (1 == code) return "HOR";
        else return "VER";
      }
    }
}