package enos.scrabble.domain;

import java.util.*;

public class Player {
  Rack theRack;
  Dictionary theDictionary = Dictionary.singleton();
  private boolean isFirstMove = false;

  public Player(Rack aRack) {
    theRack = aRack;
  }

  public Player () {
    this(new Rack());
  }

  public Object clone() {
    Rack newRack = (Rack)this.theRack.clone();
    return new Player(newRack);
  }

  public int numLettersNeeded() {
    return theRack.lettersNeeded();
  }

  public void receiveLetters(String letters) {
    theRack.addLetters(letters);
  }

  public MoveGroup getBestMove(Board theBoard) {
    List bestGroups = bestGroups(theBoard);
    if (bestGroups.size() == 0) {
      return null;
    }
    else {
      MoveGroup bestMove = bestOfList(theBoard, bestGroups);
      return bestMove;
    }
  }

  public List bestNgroups(Board theBoard, int n) {
    List bestGroups = bestGroups(theBoard);
    return bestNofList(theBoard, bestGroups, n);
  }

  public List bestGroups(Board theBoard) {
    List groups = getPossibleWords(theBoard, 500);
    List bestGroups = chooseBestGroups(theBoard, groups, 250);
    if (bestGroups.size() == 0)
      bestGroups = chooseBestGroups(theBoard, groups, 350);
    if (bestGroups.size() == 0) {
//      groups = getPossibleWords(theBoard, 500);
      bestGroups = chooseBestGroups(theBoard, groups, 450);
    }
    return bestGroups;
  }

  private List bestNofList(Board theBoard, List groups, int n) {
    Collections.sort(groups);
    List best = new ArrayList(n);
    int i = 0;
    Iterator iter = groups.iterator();
    while (iter.hasNext() &&  i < n) {
      best.add(iter.next());
      i++;
    }
    return best;
  }

  private MoveGroup bestOfList(Board theBoard, List groups) {
    return (MoveGroup)bestNofList(theBoard, groups, 1).get(0);
  }


  /**
   * get n best moves for max
   * for each move
   *    clone board and rack
   *    execute move on that board
   *    get m best moves for min on that board
   *    select move for max that gives lowest highest min value
   */

  public List getPossibleWords(Board theBoard, int numCombinations) {
    Word[] combs = (Word[])(
        theRack.bestNcombinations(numCombinations)).toArray(new Word[0]);
    Anchor[] anchors = (Anchor[])(theBoard.getAnchors()).toArray(new Anchor[0]);
    Collection words = new HashSet();
    if (anchors.length > 0) {
//      System.out.println("have anchors");
      isFirstMove = false;
      for (int i = 0; i < combs.length; i++) {
        if (combs[i].length() > 0) {
          for (int j = 0; j < anchors.length; j++) {
            Word possible = new Word(combs[i], anchors[j]);
            List someWords = theDictionary.getCandidates(possible);
            if (someWords != null)
              words.addAll(someWords);
          }
        }
      }
    } else {
      isFirstMove = true;
      for (int i = 0; i < combs.length; i++) {
          List someWords = theDictionary.getCandidates(combs[i]);
          if (someWords != null)
            words.addAll(someWords);
        }
    }
    List wordList = new ArrayList(words);
    Collections.sort(wordList, Rack.SCORE_COMPARATOR);
    return wordList;
  }

  public void useLetters(Move theMove) {
    String letters = theMove.newLettersUsed();
    theRack.removeLetters(letters);
  }

  private List chooseBestGroups(Board theBoard, List possibleWords, int numToExamine) {
    List groups = new ArrayList();
    int numWords = possibleWords.size();
    int count = 0;

    while ( count < numWords && count < numToExamine) {
      Word aWord = (Word)possibleWords.get(count++);
      Move aMove = null;
      if (isFirstMove) {
        aMove = new Move(aWord, Move.HORIZ, 7, 7);
      } else {
        aMove = new Move(aWord);
      }
      MoveGroup theGroup = null;
      try {
        if (null != (theGroup = theBoard.getCollateralMoves(aMove))) {
          if (theBoard.isValid(theGroup.originalMove()))
            groups.add(theGroup);
        }
      }catch (InvalidMoveException e) {
          // just go on to the next one; we're trying possibilities here
        }
      }
    return groups;
  }

  public String readRack() {
    return theRack.toString();
  }

  public static void main(String[] args) {
        Board theBoard = null;
    try {
      theBoard = new Board();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    char[][] b = new char[15][15];
/*    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't'; b[7][8] = 'h'; b[7][9] = 'e';
    b[8][7] = 'e';                b[8][9] = 'n';
                                  b[9][9] = 'd';
*/
    b[7][6] ='k'; b[7][7] ='a'; b[7][8] ='f';
    theBoard.replaceLetters(b);
    Rack theRack = new Rack();
//    theRack.addLetters("dehiprs");
    theRack.addLetters("eeloory");
//    theRack.addLetters("dehipr*");

//    theRack.addLetters("dehipr{");
//    theRack.addToRack("de{i{rs");
//    theRack.addToRack("deirs");

    Player thePlayer = new Player(theRack);
    MoveGroup group = thePlayer.getBestMove(theBoard);
    System.out.println(group.originalMove().getString());
  }
}