package enos.scrabble.domain;

import java.util.*;

public class Rack {
  private int[] letterCounts = new int[27];
  private static boolean[][] perms = new boolean[127][7];
  private int numLetters = 0;
  public static final LetterScoreComparator SCORE_COMPARATOR =
      new LetterScoreComparator();
  // static initiallizer
  { buildPerms(); }

  public Rack() {
  }

  public Object clone() {
    Rack newRack = new Rack();
    int[] theLetters = (int[])letterCounts.clone();
    newRack.letterCounts = theLetters;
    newRack.numLetters = this.numLetters;
    return newRack;
  }

  public void addLetters(String lets) {
//    System.out.println("Adding letters " + lets);
    char[] letters = lets.toLowerCase().toCharArray();
    for (int i = 0; i < letters.length; i++) {
      if (letters[i] == '{') {
        letterCounts[26]++;
//        System.out.println("added <" + "{> at index <26>");
      }
      else
      if (letters[i] != 0) {
        letterCounts[letters[i] - 'a']++;
//        System.out.println("added <" + letters[i] + "> at index <" +
//                           (int)(letters[i] - 'a'));
      }
    }
    numLetters += lets.length();
  }

  public void removeLetters(String lets) {
    char[] letters = lets.toCharArray();
    for (int i = 0; i < letters.length; i++) {
      if (letters[i] == '*' || letters[i] == '{' || letters[i] <= 'Z') {
        letterCounts[26]--;
      }
      else
      if (letters[i] != 0) {
        letterCounts[letters[i] - 'a']--;
      }
    }
    numLetters -= lets.length();
  }

  public int lettersNeeded() {
    return 7 - numLetters;
  }

  public List bestNcombinations(int n) {
    List allCombs = getCombinations();
    if (letterCounts[26] > 0) {
      allCombs = handleBlanks(allCombs);
    }
//    System.out.println("number of combs->" + allCombs.size());
    // sort based on wordPotential
    Collections.sort(allCombs, new WordPotentialComparator());

    List bestCombs = new ArrayList(n);
    int numbCombs = allCombs.size();
    for (int i = 0; i < n && i < numbCombs; i++) {
      bestCombs.add(allCombs.get(i));
    }
    return bestCombs;
  }

  private List handleBlanks(List combs) {
    List betterCombs = new ArrayList();
    Iterator iter = combs.iterator();
    while (iter.hasNext()) {
      Word theWord = (Word)iter.next();
      if (theWord.containsBlank()) {
//        iter.remove();
        String originalLetters = theWord.lettersWithoutBlank();
        for (int i = 0; i < 26; i++) {
          String origPlusOne = originalLetters + ((char)('A' + i));
//          System.out.println(origPlusOne);
          if ((originalLetters.length() + 2) == theWord.length()) {
            // two blanks!!
            for (int j = 0; j < 26; j++) {
              String origPlusTwo = origPlusOne + ( (char) ('A' + i));
              betterCombs.add(new Word(origPlusTwo));
            }
          } else {
            betterCombs.add(new Word(origPlusOne));
          }
        }
      } else {
        betterCombs.add(theWord);
      }
    }
    return betterCombs;
  }

  private static void buildPerms() {
    List permList = new ArrayList(127);
    for (int i = 1; i < 128; i++) {
//      System.out.println(Integer.toBinaryString(i));
      char[] charmap = (Integer.toBinaryString(i)).toCharArray();
      int targetBit = 6;
      boolean[] thisBitmap = new boolean[7];
      for (int j = charmap.length; j > 0; j--) {
        if ('1' == charmap[j - 1]) {
          thisBitmap[targetBit] = true;
        }
        targetBit--;
      }
      permList.add(new PermMatrix(thisBitmap, i));
    }
    Collections.sort(permList);
    Iterator iter = permList.iterator();
    int count = 126;
    while (iter.hasNext()) {
      PermMatrix item = (PermMatrix) iter.next();
      perms[count--] = item.perm;
    }
//    for (int i = 0; i < 127; i++) {
//      for (int k = 0; k < 7; k++) {
//        char bit = perms[i][k] ? '1' : '0';
//        System.out.print(bit);
//      }
//      System.out.println();
//    }
  }

  public List getCombinations() {
    List combs = new ArrayList(128);
    char[] letters = getCharArray();
    for (int i = 0; i < 127; i++) {
      StringBuffer buf = new StringBuffer(7);
      //printMatrix(perms[i]);
      for (int j = 0; j < 7; j++) {
        if (perms[i][j] &&
            ( (letters[j] >= 'a' && letters[j] <= '{'))) {
          //System.out.println("appending: " + letters[j]);
          buf.append(letters[j]);
        }
      }
      //System.out.println(buf.toString());
      String str = buf.toString();
      if (str != null) {
        combs.add(new Word(buf.toString()));
      }
    }
    return combs;
  }

  private void printMatrix(boolean[] mat) {
    System.out.print("matrix: ");
    for (int k = 0; k < 7; k++) {
      char bit = mat[k] ? '1' : '0';
      System.out.print(bit);
    }
    System.out.println();
  }

  private char[] getCharArray() {
//    System.out.println("\nWorking with rack:");
    char[] letters = new char[7];
    int index = 0;
    for (int i = 0; i < 27; i++) {
      for (int j = 0; j < letterCounts[i]; j++) {
        letters[index++] = (char) ('a' + i);
//        System.out.print(letters[index-1]);
      }
    }
//    System.out.println();
//    for (int j = 0; j < letterCounts[26]; j++)
//      letters[6 - j] = '[';
    return letters;
  }

  public static void main(String[] args) {
    Rack theRack = new Rack();
    theRack.addLetters("knarfle");
 //   List combs = theRack.getCombinations();
    List combs = theRack.bestNcombinations(50);
    System.out.println(combs.size());
    Iterator iter = combs.iterator();
    while (iter.hasNext()) {
      System.out.println(iter.next());
    }
  }

  public static class PermMatrix implements Comparable {
    boolean[] perm;
    int decimal;
    int index = -1;
    private PermMatrix(boolean[] thePerm, int theInt) {
      perm = thePerm;
      decimal = theInt;
    }

    private int index() {
      if ( -1 == index) {
        index = 0;
        for (int i = 0; i < perm.length; i++) {
          if (perm[i]) {
            index++;
          }
        }
      }
      return index;
    }

    public int compareTo(Object o) {
      if (this == o) {
        return 0;
      }
      final PermMatrix that = (PermMatrix) o;
      if (this.index() < that.index()) {
        return -1;
      }
      if (this.index() > that.index()) {
        return 1;
      }
      if (this.index() == that.index()) {
        if (this.decimal < that.decimal) {
          return -1;
        }
        if (this.decimal > that.decimal) {
          return 1;
        }
        return 0;
      }
      return 0;
    }

    public boolean equals(Object other) {
      if (other == this) {
        return true;
      }
      if (! (other instanceof PermMatrix)) {
        return false;
      }
      PermMatrix that = (PermMatrix) other;
      if (this.decimal == that.decimal) {
        return true;
      }
      else {
        return false;
      }
    }
  }

  public String toString() {
    StringBuffer letts = new StringBuffer();
    for (int i = 0; i < letterCounts.length; i++) {
      for (int j = 0; j < letterCounts[i]; j++) {
        letts.append((char)('a' + i));
      }
    }
    String theString = letts.toString();
    theString.replaceAll("\\{", "*");
    return theString;
  }

  public static class LetterScoreComparator implements Comparator {
      public int compare(Object o1, Object o2) {
        Word w1 = (Word) o1;
        Word w2 = (Word) o2;
        return ( (w1.totalLetterScores() > w2.totalLetterScores()
                  ? -1
                  : w1.totalLetterScores() < w2.totalLetterScores()
                  ? 1
                  : 0));
      }
    }

  public static class WordPotentialComparator implements Comparator {
      public int compare(Object o1, Object o2) {
        Word w1 = (Word) o1;
        Word w2 = (Word) o2;
        return ( (w1.getWordPotential() > w2.getWordPotential()
                  ? -1
                  : w1.getWordPotential() < w2.getWordPotential()
                  ? 1
                  : 0));
      }
    }
}