package enos.scrabble.test;

import junit.framework.*;
import enos.scrabble.domain.*;
import enos.scrabble.util.*;
import java.util.Collection;
import java.util.List;
import java.util.ArrayList;
import java.util.Iterator;
import java.io.IOException;


public class BoardTest extends  TestCase {
  public BoardTest(String name) {
    super(name);
  }

  public void testScore() throws Exception {
    Board theBoard = new Board();
    Word w1 = new Word("chickens");
    Move chicken = new Move(w1, Move.HORIZ, 0, 0);
    int actual = theBoard.setScore(chicken);
    int expected = 198;
    assertEquals("chickens", expected, actual);
    chicken.signalSquaresUsed();
    Word w2 = new Word("clown");
    Move clown = new Move(w2, Move.VERT, 0, 0);
    actual = theBoard.setScore(clown);
    expected = 14;
    assertEquals("clown", expected, actual);
  }

  public void testRack() {
    Dictionary dict = Dictionary.singleton();
    List candidates = dict.getCandidates(new Word("knarf"));
    assertEquals(1, candidates.size());
    assertEquals(new Word("frank"), candidates.get(0));

    candidates = dict.getCandidates(new Word("knarf"));
    assertEquals(1, candidates.size());
    assertEquals(new Word("frank"), candidates.get(0));
  }

  public void testBigram() throws Exception {
    Word aer = new Word("aer");
    BigramEvaluator eval = BigramEvaluator.singleton();
    eval.scoreBigrams(aer);
    float expected = 1.13643f;
    assertEquals(expected, aer.getBigramScore(), 0.00000001f);

    Word aa = new Word("aa");
    float aaExpected = 0.10561f;
    eval.scoreBigrams(aa);
    assertEquals(aaExpected, aa.getBigramScore(), 0.0000001f);
  }

  public void testLetterScore() {
    Word abe = new Word("abe");
    assertEquals(5, abe.totalLetterScores());
  }

  public void xtestCombs() {
    Rack theRack = new Rack();
    theRack.addLetters("knarfle");
    List combs = theRack.getCombinations();
//    List combs = theRack.bestNcombinations(20);
    System.out.println(combs.size());
    Iterator iter = combs.iterator();
    while (iter.hasNext()) {
      System.out.println(iter.next());
    }
  }

  public void testAnchors() {
    Board theBoard = new Board();
    char[][] b = new char[15][15];
    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't'; b[7][8] = 'h'; b[7][9] = 'e';
    b[8][7] = 'e';                b[8][9] = 'n';
                                  b[9][9] = 'd';
    theBoard.replaceLetters(b);
//    Board theBoard = (Board)originalBoard.clone();
    List expectedAnchors = new ArrayList();
    expectedAnchors.add(new Anchor("l", 7, 5, Move.HORIZ));
    expectedAnchors.add(new Anchor("a", 7, 6, Move.HORIZ));
    expectedAnchors.add(new Anchor("the", 7, 7, Move.HORIZ));
    expectedAnchors.add(new Anchor("e", 7, 8, Move.HORIZ));
    expectedAnchors.add(new Anchor("n", 9, 8, Move.HORIZ));
    expectedAnchors.add(new Anchor("d", 9, 9, Move.HORIZ));
    expectedAnchors.add(new Anchor("late", 7, 5, Move.VERT));
    expectedAnchors.add(new Anchor("h", 8, 7, Move.VERT));
    expectedAnchors.add(new Anchor("end", 9, 7, Move.VERT));
    List theAnchors = theBoard.getAnchors();
    assertEquals(9, theAnchors.size());
    assertTrue(theAnchors.containsAll(expectedAnchors));
  }

  public void testBigramWithBlank() {
    Word blank = new Word("{");
    float score = blank.getBigramScore();
    assertEquals(.5f, score, 0f);
  }

  public void testMoveScores() throws InvalidMoveException {
    Board theBoard = new Board();
    char[][] b = new char[15][15];
    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't'; b[7][8] = 'h'; b[7][9] = 'e';
    b[8][7] = 'e';                b[8][9] = 'n';
                                  b[9][9] = 'd';
    Move the = new Move(new Word("the"), Move.HORIZ, 7, 7);
    MoveGroup group = theBoard.getCollateralMoves(the);
    int actual1 = group.score();
    int expected1 = 12;
    assertEquals("the", expected1, actual1);
    theBoard.executeMove(group);

    Move end = new Move(new Word("end"), Move.VERT, 9, 7);
    assertTrue("end valid", theBoard.isValid(end));
    group = theBoard.getCollateralMoves(end);
    int actual2 = group.score();
    int expected2 = 8;
    assertEquals("end", expected2, actual2);
    theBoard.executeMove(group);
    /////*****************************************

    Move late = new Move(new Word("late"), Move.VERT, 7, 5);
    assertTrue("late valid", theBoard.isValid(late));
    group = theBoard.getCollateralMoves(late);
    int actual3 = group.score();
    int expected3 = 4;
    assertEquals("late", expected3, actual3);
    theBoard.executeMove(group);

    Move sheen = new Move(new Word("sheen"), Move.HORIZ, 5, 8);
    assertTrue("sheen valid", theBoard.isValid(sheen));
    group = theBoard.getCollateralMoves(sheen);
    int actual4 = group.score();
    int expected4 = 19;
    assertEquals("sheen", expected4, actual4);
    theBoard.executeMove(group);
  }

  public void testPlayer() {
    Board theBoard = new Board();
    char[][] b = new char[15][15];
    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't'; b[7][8] = 'h'; b[7][9] = 'e';
    b[8][7] = 'e';                b[8][9] = 'n';
                                  b[9][9] = 'd';
    theBoard.replaceLetters(b);
    Rack theRack = new Rack();
//    theRack.addToRack("dehiprs");
    theRack.addLetters("dehipr{");
//    theRack.addToRack("de{i{rs");
//    theRack.addToRack("deirs");

    Player thePlayer = new Player(theRack);
    Collection words = thePlayer.getPossibleWords(theBoard, 250);
    Iterator iter = words.iterator();
    float scores = 0f;
    float count =0f;
    while (iter.hasNext()) {
      Word theWord = (Word) iter.next();
      System.out.println(theWord +"->" + theWord.totalLetterScores());
      scores += theWord.totalLetterScores();
      count++;
    }
    System.out.println("scores------> "+ scores);
    System.out.println(count);
    float mean = scores/count;
    System.out.println("mean------> "+ mean);
  }


  public void testBestMove() {
    Board theBoard = null;
    theBoard = new Board();
    char[][] b = new char[15][15];
    b[5][7] = 'l';
    b[6][7] = 'a';
    b[7][7] = 't'; b[7][8] = 'h'; b[7][9] = 'e';
    b[8][7] = 'e';                b[8][9] = 'n';
                                  b[9][9] = 'd';
    theBoard.replaceLetters(b);
    Rack theRack = new Rack();
//    theRack.addToRack("dehiprs");
    theRack.addLetters("dehipr{");
//    theRack.addToRack("de{i{rs");
//    theRack.addToRack("deirs");

    Player thePlayer = new Player(theRack);
    MoveGroup group = thePlayer.getBestMove(theBoard);
    System.out.println(group.originalMove().getString());
  }

}