package enos.scrabble.test;

import enos.scrabble.domain.*;
import enos.scrabble.datastruct.*;
import junit.framework.*;
import java.util.*;

public class HashTest extends TestCase {

  public HashTest(String name) {
    super(name);
  }

  public void testToString() {
    Word dog = new Word("dog");
    Word Dog = new Word ("Dog");
    assertEquals(dog.toString(), "dog");
    assertEquals(Dog.toString(), "Dog");
  }

  public void testHashCode() {
    Word dog = new Word("dog");
    Word Dog = new Word ("Dog");
    Word god = new Word("god");
    Word good = new Word("good");
    int expectedHash = 16384 + 64 + 8;
    assertEquals("dog", expectedHash, dog.hashBinary());
    assertEquals("Dog", expectedHash, Dog.hashBinary());
    assertEquals("god",expectedHash, god.hashBinary());
    assertEquals("good", expectedHash, good.hashBinary());
  }

  public void testWordEquals() {
    Word dog1 = new Word("dog");
    Word Dog = new Word ("Dog");
    Word god = new Word("god");
    Word dog2 = new Word("dog");
    Word dog3 = new Word(new String("dog"));
    assertTrue("2 dogs", dog1.wordEquals(dog2));
    assertTrue("Dog", dog1.wordEquals(Dog));
    assertTrue("silly constructor", dog1.wordEquals(dog3));
    assertTrue("3rd pair", dog2.wordEquals(dog3));
    assertTrue("dog, god", !god.wordEquals(dog1));
  }


  public void testSameLetters() {
    Word dog = new Word("dog");
    Word Dog = new Word ("Dog");
    Word dgo = new Word("dgo");
    Word dog1 = new Word("dog");
    Word god = new Word("god");
    Word good = new Word("good");
    assertTrue("2 dogs", dog.sameLetters(dog1));
    assertTrue("dog, Dog", dog.sameLetters(Dog));
    assertTrue("dog, god", dog.sameLetters(god));
    assertTrue("dog, dgo", dog.sameLetters(dgo));
    assertTrue("wrong", !dog.sameLetters(good));
  }

  public void testExists() {
    Word dog = new Word("dog");
    Word dgo = new Word("dgo");
    Word dog1 = new Word("dog");
    Word god = new Word("god");
    Word good = new Word("good");
    Word zipper = new Word("zipper");
    HashOfWords hash = new HashOfWords();
    hash.add(dog);
    hash.add(dog1);
    hash.add(god);
    hash.add(good);
    hash.add(zipper);
    assertTrue("dog", hash.exists(new Word("dog")));
    assertTrue("god", hash.exists(new Word("god")));
    assertTrue("good", hash.exists(new Word("good")));
    assertTrue("zipper", hash.exists(new Word("zipper")));
    assertTrue("dgo", !hash.exists(new Word("dgo")));
    assertTrue("froob", !hash.exists(new Word("froob")));
  }

  public void testMatches() {
    Word dog = new Word("dog");
    Word dgo = new Word("dgo");
    Word dog1 = new Word("dog");
    Word god = new Word("god");
    Word good = new Word("good");
    Word zipper = new Word("zipper");
    HashOfWords hash = new HashOfWords();
    hash.add(dog);
    hash.add(dog1);
    hash.add(god);
    hash.add(good);
    hash.add(zipper);
    List expectedContents = new ArrayList();
    expectedContents.add(dog);
    expectedContents.add(god);
    List matches = hash.getMatches(dgo);
    assertTrue("matches", matches.containsAll(expectedContents));
    assertTrue("size", matches.size() == 2);

    Word dgoWiAnchor = new Word("dgo");
    Anchor og = new Anchor("og", 7,7, Move.HORIZ);
    dgoWiAnchor.addAnchor(og);
    expectedContents = new ArrayList();
    expectedContents.add(dog);
    matches = hash.getMatches(dgoWiAnchor);
    assertTrue("matches", matches.containsAll(expectedContents));
    assertTrue("with anchor size", matches.size() == 1);


    List bucket = hash.getBucket(dgo);
    List expectedBucket = new ArrayList();
    expectedBucket.add(dog);
    expectedBucket.add(god);
    expectedBucket.add(good);
    assertTrue("bucket contents", bucket.containsAll(expectedBucket));
    assertTrue("bucket size", bucket.size() == 3);

    Word gloob = new Word("gloob");
    assertNull(hash.getMatches(gloob));
  }
}