package enos.scrabble.util;

import java.io.*;
import enos.scrabble.domain.*;
import java.util.*;
import java.net.URL;

public class BigramEvaluator {
  private List bigrams = new ArrayList();
  public static BigramEvaluator singleton;


  public static BigramEvaluator singleton() {
    if (null == singleton) {
      try {
        singleton = new BigramEvaluator();
      }catch (IOException e) {
        e.printStackTrace();
        throw new RuntimeException("Couldn't initialize BigramEvaluator. It's" +
                                   " all over for me.");
      }
    }
    return singleton;
  }

  private BigramEvaluator() throws IOException {
    init();
  }

  public void scoreBigrams(Word theWord) {
    Iterator iter = bigrams.iterator();
    while (iter.hasNext()) {
      Bigram theBigram = (Bigram)iter.next();
      if (theBigram.isIn(theWord)) {
          float theProb = theBigram.prob;
//          if(theWord.toString().indexOf("{") > -1) System.out.println("found {");
//          if (theProb == .5f) System.out.println("matched {");
          theWord.increaseBigramScore(theBigram.prob);
      }
    }
  }

  private void init() throws IOException {
    BufferedReader reader = null;
    File file = new File(Game.bigramFile);
    if (file.exists()) {
      System.out.println("init via file");
      reader = new BufferedReader(new FileReader(file));
    }else {
      URL url = getClass().getResource("/" + Game.bigramFile);
      InputStreamReader isReader = new InputStreamReader(url.openStream());
      reader = new BufferedReader(isReader);
    }
    String line = null;
    while (null != (line = reader.readLine())) {
      StringTokenizer tok = new StringTokenizer(line);
      Bigram theBigram = new Bigram(tok.nextToken(), tok.nextToken(),
                                    Float.parseFloat(tok.nextToken()));
      bigrams.add(theBigram);
    }
    reader.close();
    // favor the blank square
    bigrams.add(new Bigram("{", "</s>", .5f));
  }

  private class Bigram {
    float prob;
    char[] pair = new char[2];

    private Bigram(String lexeme1, String lexeme2, float theProb) {
      if (lexeme1.equals("<s>"))
        ; //skip
      else pair[0] = lexeme1.charAt(0);

      if (lexeme2.equals("</s>"))
        ; //skip
      else pair[1] = lexeme2.charAt(0);

      prob = theProb;
    }

    protected boolean isIn(Word theWord) {
      String theString = theWord.toString().toLowerCase();
      if (pair[0] == pair[1]) {
        return (theString.indexOf(pair[0]) != (theString.lastIndexOf(pair[1])));
      } else {
        return ( (0 == pair[0] || theString.indexOf(pair[0]) > -1) &&
                ( (0 == pair[1] || theString.indexOf(pair[1]) > -1)));
      }
    }

    public String toString() {
      return (pair[0] + "") + (pair[1] + "") +  "->" + prob;
    }
  }

  public static void main(String[] args) {
    BigramEvaluator eval = null;
    try {
      eval = new BigramEvaluator();
    }
    catch (IOException ex) {
      ex.printStackTrace();
    }
    Iterator iter = eval.bigrams.iterator();
    while (iter.hasNext()) {
      System.out.println(iter.next().toString());
    }
  }

}