package enos.scrabble.util;
import enos.scrabble.domain.*;
import java.util.*;

public class MiniMaximizer {
  int max;
  int min;

  public MiniMaximizer() {
  }

  public Move miniMax(Game aGame, int theMax, int theMin, int numberOfPlies){
    max = theMax;
    min = theMin;
    Game gameClone = (Game)aGame.clone();
    MoveGroup bestGroup = maxValue(gameClone, -999999, 999999, numberOfPlies - 1);
    if (null != bestGroup) {
      return bestGroup.originalMove();
    } else {
      return null;
    }
  }

  private MoveGroup maxValue(Game aGame, int alpha, int beta, int pliesToGo) {
    if (0 == pliesToGo) {
      MoveGroup best = aGame.getBestMoveGroup(max);
      if (null != best) {
        best.setUtility(best.score());
      }
      return best;
    } else {
      int lettersNeeded = aGame.numLettersNeeded(max);
      if (lettersNeeded  > 0) {
        String newLetters = aGame.getNletters(lettersNeeded);
        if (newLetters != null) {
          aGame.giveLettersToPlayer(newLetters, max);
        }
      }
      List possible = aGame.getNbestMoveGroups(max, 20);
      Iterator iter = possible.iterator();
      MoveGroup best = null;
      int maxV = -999999;
      while (iter.hasNext()) {
        MoveGroup aMove = (MoveGroup)iter.next();
        Game gameClone = (Game)aGame.clone();
        try {
          gameClone.move(aMove.originalMove(), max);
        } catch(InvalidMoveException e) {continue;}
        MoveGroup current = minValue(gameClone, alpha, beta, pliesToGo - 1);
        if (null != current && (current.getUtility() + aMove.score()) > maxV) {
          maxV = current.getUtility() + aMove.score();
          best = aMove;
          if (maxV >= beta) {
            if (null != best) {
              best.setUtility(best.score() + maxV);
              return best;
            }
          }
          if (alpha < maxV) {
            alpha = maxV;
          }
        }
      }
      if (null != best) {
        best.setUtility(maxV);
      }
      return best;
    }
  }

  private MoveGroup minValue(Game aGame, int alpha, int beta, int pliesToGo) {
    if (0 == pliesToGo) {
      MoveGroup worst = aGame.getBestMoveGroup(min);
      if ( null != worst) {
        worst.setUtility(0 - worst.score());
      }
      return worst;
    } else {
      int lettersNeeded = aGame.numLettersNeeded(min);
      if (lettersNeeded  > 0) {
        String newLetters = aGame.getNletters(lettersNeeded);
        if (newLetters != null) {
          aGame.giveLettersToPlayer(newLetters, min);
        }
      }
      List possible = aGame.getNbestMoveGroups(min, 20);
      Iterator iter = possible.iterator();
      MoveGroup worst = null;
      int minV = 999999;
      while (iter.hasNext()) {
        MoveGroup aMove = (MoveGroup)iter.next();
        Game gameClone = (Game)aGame.clone();
        try {
          gameClone.move(aMove.originalMove(), min);
        } catch (InvalidMoveException e) { continue; };
        MoveGroup current = maxValue(gameClone, alpha, beta, pliesToGo - 1);
        if (null != current && (current.getUtility()-aMove.score()) < minV) {
          minV = current.getUtility() - aMove.score();
          worst = aMove;
          if (minV <= alpha) {
            if (null != worst) {
              worst.setUtility(minV);
              return worst;
            }
          }
          if (beta > minV) {
            beta = minV;
          }
        }
      }
      if (null != worst) {
        worst.setUtility(worst.getUtility() + minV);
      }
      return worst;
    }
  }

}
